@@ -452,6 +452,8 @@ merge_hwe (struct hwentry * dst, struct hwentry * src)
 	merge_num(retain_hwhandler);
 	merge_num(detect_prio);
 	merge_num(detect_checker);
+	merge_num(detect_pgpolicy);
+	merge_num(detect_pgpolicy_use_tpg);
 	merge_num(deferred_remove);
 	merge_num(delay_watch_checks);
 	merge_num(delay_wait_checks);
@@ -617,6 +619,8 @@ store_hwe (vector hwtable, struct hwentry * dhwe)
 	hwe->retain_hwhandler = dhwe->retain_hwhandler;
 	hwe->detect_prio = dhwe->detect_prio;
 	hwe->detect_checker = dhwe->detect_checker;
+	hwe->detect_pgpolicy = dhwe->detect_pgpolicy;
+	hwe->detect_pgpolicy_use_tpg = dhwe->detect_pgpolicy_use_tpg;
 	hwe->ghost_delay = dhwe->ghost_delay;
 	hwe->vpd_vendor_id = dhwe->vpd_vendor_id;
 
@@ -748,15 +752,6 @@ static void _uninit_config(struct config *conf)
 	if (conf->hwhandler)
 		free(conf->hwhandler);
 
-	if (conf->bindings_file)
-		free(conf->bindings_file);
-
-	if (conf->wwids_file)
-		free(conf->wwids_file);
-
-	if (conf->prkeys_file)
-		free(conf->prkeys_file);
-
 	if (conf->prio_name)
 		free(conf->prio_name);
 
@@ -918,9 +913,6 @@ int _init_config (const char *file, struct config *conf)
 	 * internal defaults
 	 */
 	get_sys_max_fds(&conf->max_fds);
-	conf->bindings_file = set_default(DEFAULT_BINDINGS_FILE);
-	conf->wwids_file = set_default(DEFAULT_WWIDS_FILE);
-	conf->prkeys_file = set_default(DEFAULT_PRKEYS_FILE);
 	conf->attribute_flags = 0;
 	conf->reassign_maps = DEFAULT_REASSIGN_MAPS;
 	conf->checkint = CHECKINT_UNDEF;
@@ -1074,12 +1066,6 @@ int _init_config (const char *file, struct config *conf)
 	merge_blacklist(conf->elist_wwid);
 	merge_blacklist_device(conf->elist_device);
 
-	if (conf->bindings_file == NULL)
-		conf->bindings_file = set_default(DEFAULT_BINDINGS_FILE);
-
-	if (!conf->bindings_file || !conf->wwids_file || !conf->prkeys_file)
-		goto out;
-
 	libmp_verbosity = conf->verbosity;
 	return 0;
 out:
